@@ -18,6 +18,9 @@ class RealResponse(object):
   def read(self, chunk_size=None):
     return self.response.read(chunk_size)
 
+  def close(self):
+    return self.response.close()
+
   def getheader(self, key):
     return self.response.getheader(key)
 
@@ -39,6 +42,9 @@ class StubResponse(object):
   def read(self, chunk_size=None):
     return self.__body.read(chunk_size)
 
+  def close(self):
+    pass
+
   def getheader(self, key):
     return None
 
@@ -53,7 +59,7 @@ class Request(object):
     self.__underlying = urllib.request.Request(**kwargs)
 
   def add_header(self, key, value):
-    self.__underlying.add_header(key, value)
+    self.__underlying.add_unredirected_header(key, value)
 
   @property
   def data(self):
