@@ -5,6 +5,7 @@ import tarfile
 import tempfile
 import functools
 import os
+import shutil
 import sys
 import json
 import io
@@ -16,8 +17,30 @@ from .http import Request
 
 class Docker(object):
 
-  @staticmethod 
+  @staticmethod
+  def __print(msg):
+    sys.stdout.write("\r\033{}{}".format(msg, ' '*100))
+    sys.stdout.flush()    
+
+  @staticmethod
+  def __progress_bar(ublob, done, total):
+    nb_traits = int(50*done/total)
+    sys.stdout.write('\r' + ublob + ': Downloading [')
+    for i in range(0, nb_traits):
+      if i == nb_traits - 1:
+        sys.stdout.write('>')
+      else:
+        sys.stdout.write('=')
+    for i in range(0, 49 - nb_traits):
+      sys.stdout.write(' ')
+    sys.stdout.write('] {}/{}'.format(done, total))
+    sys.stdout.flush()
+
+  @staticmethod
   def __get_auth_head(repository):
+    sys.stdout.write("\r\033{}: Authenticating...".format(repository))
+    sys.stdout.flush()
+
     uri = 'https://auth.docker.io/token?service=registry.docker.io&scope=repository:{}:pull'.format(repository)
     
     request = Request(method='GET', url=uri)
@@ -26,8 +49,11 @@ class Docker(object):
     assert response.status == 200, 'unable to authorize docker pull for {} with {} {}'.format(repository, response.status, response.read().decode('utf-8'))
     
     data = json.loads(response.read().decode('utf-8'))
+
+    sys.stdout.write("\r\033[K")
+    sys.stdout.flush()
     return {
-      'Authorization':'Bearer {}'.format(data['token']),
+      'Authorization': 'Bearer {}'.format(data['token']),
       'Accept': 'application/vnd.docker.distribution.manifest.v2+json'
     }
 
@@ -51,9 +77,11 @@ class Docker(object):
 
   @staticmethod
   def extract_file(repository, layer, source, target):
-    print(layer)
-    file = source.strip(os.path.sep)
-    if len(layer['urls']):
+    tag = repository + ':sha256(' + layer['digest'][7:19] + ')'
+
+    Docker.__print('{}: Downloading...'.format(tag))
+
+    if len(layer.get('urls', [])):
       uri = layer['urls'][0]
     else:
       uri = 'https://index.docker.io/v2/{}/blobs/{}'.format(repository, layer['digest'])
@@ -66,38 +94,44 @@ class Docker(object):
 
     assert response.status == 200, 'unable to download layer {}:{} with {} {}'.format(repository, layer['digest'], response.status, response.read().decode('utf-8'))
 
-    fileobj = io.BytesIO()
-    content_length = response.getheader("Content-Length")
+    total_size = int(response.getheader('Content-Length'))
+    block_size = max(int(total_size/1000), 1024**2)
+    downloaded_size = 0
 
-    if content_length is not None:
-      try:
-        content_length = int(content_length)
-      except ValueError:
-        content_length = None
-      if content_length:
-        fileobj.seek(content_length - 1)
-        fileobj.write(b"\0")
-        fileobj.seek(0)
+    fileobj = tempfile.TemporaryFile()
 
+    Docker.__progress_bar(tag, 0, total_size)
     while 1:
-      buf = response.read(4 * 1024)
-      if not buf:
+      block = response.read(block_size)
+      if not block:
         break
-      fileobj.write(buf)
+      fileobj.write(block)
+      downloaded_size += len(block)
+      Docker.__progress_bar(tag, downloaded_size, total_size)
 
-    fileobj.seek(0)
+    Docker.__print('{}: Downloaded Layer...'.format(tag))
 
-    tarf = tarfile.open(fileobj=fileobj, mode='r:gz')
+    response.close()
+
+    file = source.strip(os.path.sep)
+    Docker.__print('{}: Scanning Layer for {}...'.format(tag, file))
+
+    tarf = tarfile.open(fileobj.name, mode='r:gz')
 
     for member in tarf.getmembers():
       if member.name != file:
         continue
-      tmp_dir = tempfile.TemporaryDirectory()
-      tarf.extract(member, path=tmp_dir.name)
-      os.rename(os.path.join(tmp_dir.name, file), os.path.join(target, os.path.basename(file)))
-      tmp_dir.cleanup()
+      Docker.__print('{}: Extracting...'.format(tag))
+      tgt = os.path.realpath(os.path.join(target, os.path.basename(file)))
+      
+      with tarf.extractfile(member) as fs:
+        with open(tgt, 'wb') as fd:
+          shutil.copyfileobj(fs, fd)
+
+      Docker.__print('Downloaded... {}\n'.format(tgt))
       return True
 
+    Docker.__print('')
     return False
 
 
